#![feature(iter_intersperse)]
#![feature(trait_alias)]
use std::io::{BufRead, Write};
use command::{Command, Env, Args};

mod command;

fn main() {
    let stdin = std::io::stdin();
    let mut in_lock = stdin.lock();

    let stdout = std::io::stdout();
    let mut out_lock = stdout.lock();
    
    let mut env= Env::new();
    env.set("HOME", "/a/fake/dir");
    env.set("PS1", "> ");

    let commands = &Command::all_commands(&env);
    println!("Loaded {} commands", commands.len());

    loop {
        let mut buf = String::new();

        let default_ps1 = "> ".to_string();
        let ps1 = env.get("PS1").unwrap_or(&default_ps1).as_bytes();
        out_lock.write(ps1).unwrap();
        out_lock.flush().unwrap();

        in_lock
            .read_line(&mut buf)
            .expect("Failed to read in to buffer.");

        let mut args: Args = &buf.trim().split(' ').map(|f| f.to_owned()).collect::<Vec<String>>();

        // let mut args: Args = &buf.trim().split(' ').collect::<Vec<&str>>();
        let mut options = commands;

        loop {
            match args.first() {
                Some(cmd_arg) => {
                    // Try to find a subcommand to run
                    match Command::choose(cmd_arg, options) {
                        Some(choice) => {
                            // No sub commands left to choose from, just pass the input
                            if choice.sub_commands.is_empty() {
                                choice.call(args, &mut env);
                                break;
                            } 

                            args = &args[1..];
                            options = &choice.sub_commands;

                            // a sort of last chance before running into hella edgecases
                            if args.is_empty()
                            || Command::choose(&args[0], options).is_none() // yes, this means that we search for a choice twice if it misses, but the search takes all of 0ms, so it's fine.
                            {
                                choice.call(args, &mut env);
                                break;
                            }
                        }
                        // When no good choice presents it's self
                        None => {
                            println!("No option...");
                            break;
                        },
                    }
                }
                None => unimplemented!("Couldn't get index 0 of the args"),
            }
        }
    }
}
